<?php

if ( !class_exists('wsPluginInfo') ):

/**
 * Class for storing and manipulating plugin compatibility & rating data.
 *
 * @package Broken Link Checker
 * @author Jnis Elsts 
 * @copyright 2010
 * @access public
 */
class wsPluginInfo {
	
	var $option_name = ''; //Load/store the data from/to this option
	var $data;
	
  /**
   * Class constructor
   *
   * @param string $option_name Load/store the plugin data from/to this option
   * @return void
   */
	function wsPluginInfo($option_name = ''){
		if ( !empty($option_name) ){
			$this->option_name = $option_name;
		}
		$this->load_data();
	}
	
  /**
   * Search the stored data and delete records pertaining to plugins that are no longer installed.
   *
   * @return void
   */
	function cleanup(){
		global $wp_version;
		
		$installed_plugins = get_plugins();
		
		$keep = array();
		foreach($this->data as $plugin => $data){
			if ( !array_key_exists($plugin, $installed_plugins) ){
				continue; //Skip uninstalled plugins
			}
			
			/*
			$plugin_data = $installed_plugins[$plugin];
			
			//Remove compatibility info about older WordPress versions
			if ( !empty($data['compatibility']) ){
				$filtered_compatibility = array();
				foreach($data['compatibility'] as $wordpress_version => $plugin_versions){
					if ( version_compare($wordpress_version, $wp_version, '<') ){
						continue;
					}
					
					//Remove compatibility info about older plugin versions
					$filtered_plugin_versions = array();
					foreach($plugin_versions as $plugin_version => $compatible){
						if ( version_compare($plugin_version, $plugin_data['Version'], '<') ){
							continue;
						}
						
						$filtered_plugin_versions[$plugin_version] = $compatible;
					}
					
					if ( !empty($filtered_plugin_versions) ){
						$filtered_compatibility[$wordpress_version] = $filtered_plugin_versions;
					}					
				}
				$data['compatibility'] = $filtered_compatibility;
			}
			//*/
			
			$keep[$plugin] = $data;
		}
		$this->data = $keep;
	}
	
  /**
   * Load the plugin data.
   *
   * @return void
   */
	function load_data(){
		$user = wp_get_current_user();
		if ( !empty($user) ){
			$this->data = get_metadata('user', $user->ID, $this->option_name, true);
		}
	    
	    if( !is_array( $this->data ) ){
	        $this->data = array();
	    }
	}	
	
  /**
   * Save the plugin data.
   *
   * @return void
   */
	function save_data(){
		$user = wp_get_current_user();
		if ( !empty($user) ){
			update_metadata('user', $user->ID, $this->option_name, $this->data);
		}
	}
	
  /**
   * Get the stored voting/rating/etc data available for a plugin.
   *
   * @param string $plugin Plugin filename.
   * @param string $detail Which piece of data to return. Defaults to returning everything.
   * @return mixed
   */
	function get_plugin_info($plugin, $detail = ''){
		$default_info = array(
			'last_checked' => 0,
			'in_directory' => null,
			'ratings' => array(
				'average_rating' => null,
				'user_rating' => null,
			),
			'compatibility' => array(),
		); 
		
		if ( isset($this->data[$plugin]) ){
			$info = $this->data[$plugin];
		} else {
			$info = array();
		}
		
		$info = array_merge($default_info, $info);
		if ( !empty($detail) ){
			if ( array_key_exists($detail, $info) ){
				return $info[$detail];
			} else {
				return null;
			}
		} else {
			return $info;
		}
	}
	
  /**
   * Update the stored plugin data.
   *
   * @param string $plugin Plugin filename. 
   * @param string $detail Which piece of data to update.
   * @param mixed $new_info New value.
   * @param bool $auto_persist Immediately save the changes to the DB. Defaults to true.
   * @return void
   */
	function set_plugin_info($plugin, $detail, $new_info, $auto_persist = true){
		if ( empty($detail) ){
			$this->data[$plugin] = $new_info;
		} else {
			if ( isset($this->data[$plugin]) ){
				$this->data[$plugin][$detail] = $new_info;
			} else {
				$this->data[$plugin] = array(
					$detail => $new_info
				);
			}
		}
		
		if ( $auto_persist ){
			$this->save_data();
		}
	}
	
  /**
   * Get the compatibility vote that this user has cast for a specific plugin/WP combination.
   *
   * @param string $plugin
   * @param string $plugin_version
   * @param string $wordpress_version Defaults to the current WordPress version.
   * @return bool|null True = 'works', false = 'broken', null = no data.
   */
	function get_current_vote($plugin, $plugin_version, $wordpress_version = null){
		global $wp_version;
		if ( is_null($wordpress_version) ){
			$wordpress_version = $wp_version;
		}
		
		$compatibility = $this->get_plugin_info($plugin, 'compatibility');
		
		if ( isset($compatibility[$wordpress_version][$plugin_version]) ){
			return (bool)($compatibility[$wordpress_version][$plugin_version]);
		} else {
			return null;
		}
	}
	
  /**
   * Save the compatibility setting of a specific plugin/WP combination.
   *
   * @param string $plugin
   * @param bool $compatible
   * @param string $plugin_version
   * @param string $wordpress_version Defaults to the current WP version.
   * @param bool $auto_persist
   * @return void
   */
	function set_current_vote($plugin, $compatible, $plugin_version, $wordpress_version = null, $auto_persist = true){
		global $wp_version;
		if ( is_null($wordpress_version) ){
			$wordpress_version = $wp_version;
		}
		
		//Compatibility is alwas stored as a boolean, except when it's null (=no info).
		if ( !is_null($compatible) ){
			$compatible = (bool)$compatible;
		}
		
		$info = $this->get_plugin_info($plugin, 'compatibility');
		
		$entry = array(
			$wordpress_version => array(
				$plugin_version => $compatible,
			),
		);
		
		$info = $this->array_merge_replace_recursive(
			$info,
			$entry
		);
		$this->set_plugin_info($plugin, 'compatibility', $info, $auto_persist);
	}
	
  /**
   * Save the current star rating of a plugin
   *
   * @param string $plugin Plugin file
   * @param float $average_rating A numeric representation of the average rating, 0-100.  
   * @param float $user_rating A numeric representation of this user's rating, 0-100.
   * @param bool $auto_persist Save the data to DB immediately. Defaults to true.
   * @return void
   */
	function set_current_rating($plugin, $average_rating = null, $user_rating = null, $auto_persist = true){
		$current = $this->get_current_rating($plugin, 'all');
		if ( !is_null($user_rating) ){
			$current['user_rating'] = $user_rating;
		}
		if ( !is_null($average_rating) ){
			$current['average_rating'] = $average_rating;
		}
		
		$this->set_plugin_info($plugin, 'ratings', $current, $auto_persist);
	}
	
  /**
   * Get the saved star rating of a plugin.
   *
   * @param string $plugin Plugin file.
   * @param string $which  Return these ratings. Valid values : 'user', 'average', 'all'.
   * @return float|array|null Either a numeric rating (0-100), an assoc. array of ratings, or null if the requested rating is not known.  
   */
	function get_current_rating($plugin, $which = 'user'){
		$current = $this->get_plugin_info($plugin, 'ratings');
		
		$key = $which . '_rating';
		if ( array_key_exists($key, $current) ){
			return $current[$key];
		} elseif ( $which == 'all' ) {
			return $current;
		} else {
			return null;
		}
	}
	
  /**
   * Check if a plugin is listed in the WordPress.org directory. 
   * Cached information will be used if available.
   *
   * @param string $plugin_file
   * @param bool $default (Optional) Return this value when there's no information about the plugin. Otherwise, try to fetch the data from WordPress.org. 
   * @return bool True if the plugin is listed in the WP.org directory, false otherwise. 
   */
	function listed_in_directory($plugin_file, $default = null){
		$listed = $this->get_plugin_info($plugin_file, 'in_directory');
		
		if ( !is_null($listed) ){
			return (bool)$listed;
		} else {
			if ( is_null($default) ){
				return $this->update_directory_status($plugin_file);
			} else {
				return $default;
			} 
		}
	}
	
  /**
   * Determine if a plugin is listed in the WordPress.org directory. 
   * The result will be saved to the cache.
   *
   * @uses wsPluginVoter::fetch_directory_page()
   *
   * @param string $plugin_file
   * @return bool True if the plugin is listed in the WP.org directory, false otherwise.
   */
	function update_directory_status($plugin_file){
		$voter = new wsPluginVoter();
		$rez = $voter->fetch_directory_page($plugin_file);
		$voter->done();
		
		if ( is_wp_error($rez) ){
			return $this->set_directory_status($plugin_file, false);			
		} else {
			return $this->set_directory_status($plugin_file, true);
		}
	}
	
  /**
   * Store plugin's directory status.
   *
   * @param string $plugin_file
   * @param bool $listed Whether the plugin is listed in the WordPress.org directory.
   * @param bool $auto_persist Automatically save the data to the DB. Defaults to true.
   * @return bool The value of $listed.
   */
	function set_directory_status($plugin_file, $listed, $auto_persist = true){
		$listed = (bool)$listed;
		$this->set_plugin_info($plugin_file, 'in_directory', $listed, $auto_persist);
		return $listed;
	}
	
	
	/**
	 * Merges any number of arrays of any dimensions, the later overwriting 
	 * previous keys, unless the key is numeric, in whitch case, duplicated 
	 * values will not be added. 
	 *
	 * The arrays to be merged are passed as arguments to the function.
	 *
	 * Fixed version. Original code taken from 
	 * http://www.php.net/manual/en/function.array-merge-recursive.php#93905
	 *
	 * @access public
	 * @return array Resulting array, once all have been merged
	 */
	function array_merge_replace_recursive() {
	    // Holds all the arrays passed
	    $params = & func_get_args ();
	    
	    // First array is used as the base, everything else overwrites on it
	    $return = array_shift ( $params );
	    
	    // Merge all arrays on the first array
	    foreach ( $params as $array ) {
	        foreach ( $array as $key => $value ) {
	            // Numeric keyed values are added (unless already there)
	            if (is_int ( $key ) && (! in_array ( $value, $return ))) {
	                if (is_array ( $value )) {
	                    $return [] = $this->array_merge_replace_recursive ( $return [$$key], $value );
	                } else {
	                    $return [] = $value;
	                }
	                
	            // String keyed values are replaced
	            } else {
	                if (isset ( $return [$key] ) && is_array ( $value ) && is_array ( $return [$key] )) {
	                    $return [$key] = $this->array_merge_replace_recursive ( $return [$key], $value );
	                } else {
	                    $return [$key] = $value;
	                }
	            }
	        }
	    }
	    
	    return $return;
	}
}

endif;

?>